import argparse
import json

import sentsplit.cli
import sentsplit.meta_data
import sentsplit.segment
import sentsplit.train


def str_to_bool(v):
  return str(v).lower() in ('yes', 'true', 't', '1')


def main(*args: str) -> None:
    parser = argparse.ArgumentParser(description=sentsplit.meta_data.description)
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s {sentsplit.meta_data.version}')
    parser.set_defaults(func=lambda x: parser.print_usage())
    subparsers = parser.add_subparsers()

    # train a new CRF model
    subparser_train = subparsers.add_parser('train', help='train a new CRF model')
    subparser_train.add_argument('-l', '--lang', required=True, help='ISO language code, e.g. "ko", "en"')
    subparser_train.add_argument('-c', '--corpus', required=True, help='path to input corpus file, where one line is one sentence')
    subparser_train.add_argument('-n', '--ngram', type=int, default=5, help='maximum ngram features for CRF model')
    subparser_train.add_argument('-o', '--output', type=str, help='path to output CRF model')
    subparser_train.add_argument('--sample_min_length', type=int, default=450, help='minimum number of characters of an input sample for CRF model')
    subparser_train.add_argument('--crf_max_iteration', type=int, default=50, help='maximum number of CRF iteration for training')
    subparser_train.add_argument('--add_depunctuated_samples', action='store_true',
                                 help='add training samples with no puctuation only appropriate for certain languages '\
                                      '(e.g. "ko", "ja") that have specfic endings for sentences')
    subparser_train.add_argument('--add_despaced_samples', action='store_true',
                                 help='add training samples that consist of concatenated sentences without whitespaces among the sentences')
    subparser_train.set_defaults(func=sentsplit.cli.sentsplit_train)

    # segment a given text file
    subparser_segment = subparsers.add_parser('segment', aliases=['split'],
                                              help='segment an input file into sentences;'\
                                                   'optional arguments can override the default values defined in the `config.py`')
    subparser_segment.add_argument('-l', '--lang', required=True, help='ISO language code, e.g. "ko", "en"')
    subparser_segment.add_argument('-i', '--input', required=True, help='path to input file to segment')
    subparser_segment.add_argument('-o', '--output', help='path to output file, default is `{input}.split`')
    subparser_segment.add_argument('--model', help='path to the CRF model')
    subparser_segment.add_argument('--ngram', type=int, help='maximum ngram for both training and segmentation')
    subparser_segment.add_argument('--mincut', type=int, help='does not segment a line if its character length is shorter than the mincut')
    subparser_segment.add_argument('--maxcut', type=int, help='segment a line if its character length exceeds the maxcut')
    subparser_segment.add_argument('--strip_spaces', type=str_to_bool, default=False,
                                   help='trim (if any) whitespaces of segmented sentences at the head and tail')
    subparser_segment.add_argument('--handle_multiple_spaces', type=str_to_bool, default=True,
                                   help='substitute multiple spaces with a single space, segment, and recover the original spaces')
    subparser_segment.add_argument('--prevent_word_split', type=str_to_bool, default=True,
                                   help='prevent from splitting a word where word boundary is denoted by white spaces; '\
                                        'ignored if the splitting position is at a punctuation')
    subparser_segment.add_argument('--segment_regexes', nargs='?', type=json.loads,
                                   help='segment at either start or end indices of the matched regex patterns defined by `segment_regexes` in JSON;'\
                                        'e.g. \'[{"name": "custom", "regex": "\\\\w+봐유~+", "at": "end"}]\'')
    subparser_segment.add_argument('--prevent_regexes', nargs='?', type=json.loads,
                                   help='do not segment at characters that are matched by `prevent_regexes` regex(es) in JSON;'\
                                        'e.g. \'[{"name": "custom", "regex": "\\\\.["\\\\\'”] "}]\'')
    subparser_segment.set_defaults(func=sentsplit.cli.sentsplit_segment)

    args = parser.parse_args()
    args.func(args)



import os
import pprint
import subprocess
import sys
from argparse import Namespace
from copy import deepcopy
from datetime import datetime

from loguru import logger
from tqdm import tqdm

from sentsplit import config
from sentsplit.segment import SentSplit
from sentsplit.train import train_crf_model


def sentsplit_train(args: Namespace) -> None:
    lang = args.lang
    corpus_path = args.corpus
    ngram = args.ngram
    output_path = args.output
    sample_min_length = args.sample_min_length
    crf_max_iteration = args.crf_max_iteration
    add_depunctuated_samples = args.add_depunctuated_samples
    add_despaced_samples = args.add_despaced_samples

    args_string = pprint.pformat(vars(args), indent=2)
    logger.info(f"Training a new CRF model:\n{args_string}")

    if output_path is None:
        corpus_basename = os.path.basename(corpus_path)
        today_str = datetime.today().strftime('%Y%m%d')
        output_path = f'./{corpus_basename}.{lang}-{ngram}-gram-{today_str}.model'

    train_crf_model(corpus_path, ngram, output_path, sample_min_length, crf_max_iteration, add_depunctuated_samples, add_despaced_samples)


def sentsplit_segment(args: Namespace) -> None:
    lang = args.lang
    input_file = args.input
    output_file = args.output
    assert os.path.isfile(input_file)
    if output_file is None:
        output_file = f'{input_file}.segment'

    override_options = vars(args)
    try:
        default_config = deepcopy(getattr(config, '{}_config'.format(lang)))
    except AttributeError:
        logger.critical(f"Unsupported language: {lang.upper()}")
        sys.exit(1)

    for k, v in default_config.items():
        if override_options[k] is not None:
            default_config[k] = override_options[k]

    sentsplit = SentSplit(lang, **default_config)

    num_lines = int(subprocess.check_output(['wc', '-l', input_file]).decode('utf8').split()[0])
    with open(input_file, 'r') as inf, open(output_file, 'w') as outf:
        cnt = 0
        for line in tqdm(inf, total=num_lines):
            line = line.rstrip('\n')
            segments = sentsplit.segment(line)
            for segment in segments:
                outf.write(f"{segment}\n")
            cnt += len(segments)

        logger.info(f"{num_lines} lines are segmented into {cnt} sentences, and saved at {output_file}")
    sentsplit.close()



import re
from functools import reduce
from typing import List


def read_lines(file_path: str) -> List[str]:
    lines = []
    with open(file_path, 'r') as inf:
        for line in inf:
            lines.append(line.rstrip('\n'))
    return lines


def write_lines(lines: List[str], file_path: str) -> None:
    with open(file_path, 'w') as outf:
        for line in lines:
            outf.write(f'{line}\n')


def split_keep_multiple_separators(string: str, separators: List[str]) -> List[str]:
    '''
    Split `string` on separator(s) but also keep the separator(s)
    Modified from `http://programmaticallyspeaking.com/split-on-separator-but-keep-the-separator-in-python.html`
    to extend to multiple separators.
    '''
    rgx_multiple_separators = '|'.join([re.escape(sep) for sep in separators])
    rgx_multiple_separators = '(' + rgx_multiple_separators + ')'
    return reduce(lambda acc, elem: acc[:-1] + [acc[-1] + elem] if (elem in separators) else acc + [elem], re.split(rgx_multiple_separators, string), [])



from copy import deepcopy

base_config = {
    'ngram': 5,
    'mincut': 5,
    'maxcut': 500,
    'strip_spaces': False,
    'segment_regexes': [
        {'name': 'after_semicolon'},
        {'name': 'ellipsis'},
        {'name': 'newline'},
    ],
    'prevent_regexes': [
        {'name': 'liberal_url'},
    ],
    'handle_multiple_spaces': True,
    'prevent_word_split': True,
}

de_config = deepcopy(base_config)
de_config['model'] = 'crf_models/de-default-25032021.model'

en_config = deepcopy(base_config)
en_config['model'] = 'crf_models/en-default-25032021.model'

fr_config = deepcopy(base_config)
fr_config['model'] = 'crf_models/fr-default-25032021.model'

it_config = deepcopy(base_config)
it_config['model'] = 'crf_models/it-default-25032021.model'

ja_config = deepcopy(base_config)
ja_config['model'] = 'crf_models/ja-default-25032021.model'
ja_config['prevent_word_split'] = False

ko_config = deepcopy(base_config)
ko_config['model'] = 'crf_models/ko-default-25032021.model'

lt_config = deepcopy(base_config)
lt_config['model'] = 'crf_models/lt-default-25032021.model'

pl_config = deepcopy(base_config)
pl_config['model'] = 'crf_models/pl-default-25032021.model'

pt_config = deepcopy(base_config)
pt_config['model'] = 'crf_models/pt-default-25032021.model'

ru_config = deepcopy(base_config)
ru_config['model'] = 'crf_models/ru-default-25032021.model'

tr_config = deepcopy(base_config)
tr_config['model'] = 'crf_models/tr-default-25032021.model'

zh_config = deepcopy(base_config)
zh_config['model'] = 'crf_models/zh-default-25032021.model'
zh_config['prevent_word_split'] = False



version = '1.0.0'
description = 'A flexible sentence segmentation library using CRF model and regex rules'
author = 'Zae Myung Kim'
author_email = 'zaemyung@gmail.com'
url = 'https://github.com/zaemyung/sentsplit'



import os
from distutils.util import convert_path

from setuptools import find_packages, setup

meta = {}
meta_path = convert_path('sentsplit/meta_data.py')
with open(meta_path) as meta_file:
    exec(meta_file.read(), meta)


def requirements():
    reqfile = 'requirements.txt'
    with open(os.path.join(os.path.dirname(__file__), reqfile)) as f:
        ret = []
        for eachline in f.read().splitlines():
            if eachline.startswith("-"):
                continue
            ret.append(eachline)
        return ret


setup(name='sentsplit',
      version=meta['version'],
      description=meta['description'],
      author=meta['author'],
      author_email=meta['author_email'],
      url=meta['url'],
      packages=find_packages(),
      entry_points={
          'console_scripts': ['sentsplit=sentsplit:main'],
      },
      include_package_data=True,
      install_requires=requirements(),
      python_requires='>=3.6',
)
